#version 150

#moj_import <matrix.glsl>

#moj_import <fancy_end_portal/end_gateway_settings.glsl>

uniform sampler2D Sampler0;
uniform sampler2D Sampler1;

uniform float GameTime;
uniform int EndPortalLayers;
uniform vec2 ScreenSize;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;

out vec4 fragColor;

const mat4 SCALE_TRANSLATE = mat4(
    0.5, 0.0, 0.0, 0.25,
    0.0, 0.5, 0.0, 0.25,
    0.0, 0.0, 1.0, 0.0,
    0.0, 0.0, 0.0, 1.0
);

mat4 end_portal_layer(float layer, float timeOffset = 0 ) {
    mat4 translate = mat4(
        1.0, 0.0, 0.0, 17.0 / layer,
        0.0, 1.0, 0.0, (2.0 + layer / 1.5) * ((GameTime - timeOffset) * 3.5),
        0.0, 0.0, 1.0, 0.0,
        0.0, 0.0, 0.0, 1.0
    );

    mat2 rotate = mat2_rotate_z(radians((layer * layer * 4321.0 + layer * 9.0) * 2.0));

    mat2 scale = mat2(
        mix(MIN_STAR_SIZE, MAX_STAR_SIZE, (layer / float(STAR_LAYERS)))
    );

    return mat4(rotate*scale)* translate * SCALE_TRANSLATE;
}

float aspect = ScreenSize.x / ScreenSize.y;

void main() {
    vec2 texCoord = (gl_FragCoord.xy / ScreenSize) * vec2(aspect, 1);

    // vec3 base = texture(Sampler0, texCoord);
    vec4 base = vec4(1) * vec4(BASE_COLOUR, 1);

    vec4 stars = base;

    for (int i = 0; i < STAR_LAYERS; i++) {
        vec4 star = texture(Sampler1, ( vec4(texCoord, 1, 1) * end_portal_layer(i + 1) ).xy);

        #if STARS_BLENDING_MODE == 0
            stars = mix(stars, star * vec4(COLORS[ clamp(i, 0, COLORS.length() - 1 ) ], 1), star.a);
        #elif STARS_BLENDING_MODE == 1
            stars += star * star.aaaa * vec4(COLORS[ clamp(i, 0, COLORS.length() - 1 ) ], 1);
        #endif
    }

    fragColor = stars;
}
